@@ -1,6 +1,6 @@
 """
 Libreria per fare grafici di fit e tabelle LaTeX a partire da dati raccolti in un file Excel
-TODO: passare dimensioni immagine giuste (plt.rcParams["figure.figsize"] = (6.4,4.8)) provare 6.2,4,5 -- 6.24136,4.68101
+TODO: graph log
 """
 __docformat__="numpy"
 
@@ -48,18 +48,23 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     titoli=["",""],
     xlabel=None,
     ylabel=None,
+    chi=True,
+    x_chi=0,
+    y_chi=0,
+    post_decimal_chi=0,
+    log_to_file=False,
     path=".",
     xscale="linear",
     yscale="linear",
     griglia_fit={
         "which" : "both",
         "ls" : "dashed",
-        "color" : "gray"
+        "color" : "powderblue"
     },
     griglia_residui={
         "which" : "both",
         "ls" : "dashed",
-        "color" : "gray"
+        "color" : "powderblue"
     },
     style_fit={"fmt" : "."},
     fit_line_color="darkorange",
@@ -68,7 +73,7 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     pad_percent=0.10,
     foglio="Foglio1",
     funzioni_modifica=[None, None],
-    fig_dimension=(6.24136,4.5)
+    fig_dimension=(6.24136,4.4)
 ):
     """
     funzione che genera:
@@ -112,6 +117,18 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         stringa da utilizzare per la generazione dei label degli assi del grafico, usare notazione LaTeX. Se None allora i nomi delle colonne
         del file excel verranno utilizzati, convertendoli automaticamente in LaTeX ed interpretando um tra parentesi quadre (default: None)
 
+    chi : bool, optional
+        se True allora sul grafico viene mostrato il valore del chi quadro e il numero di gradi di libertà (default: True)
+
+    x_chi, y_chi : int, optional
+        coordinate del punto in cui scrivere il valore del chi quadro e il numero di gradi di libertà (default: 0, 0 --> primo punto del plot)
+
+    post_decimal_chi : int, optional
+        numero di cifre dopo la virgola con cui scrivere il valore del chi quadro (default: 0)
+
+    log_to_file : bool, optional
+        if True le informazioni sulla bontà del grafico sono messe in un file di testo. If False le informazioni possono essere lette in console. (default: False)
+
     path : str, optional
         path directory di partenza (default: ".")
 
@@ -119,7 +136,7 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         tipo di scala da utilizzare per l'asse corrispondente del grafico (default: 'linear')
 
     griglia_fit, griglia_residui : dict, optional
-        dizionario contenente i parametri opzionali da passare a pyplot.grid() (default: {"which" : "both", "ls" : "dashed", "color" : "gray"})
+        dizionario contenente i parametri opzionali da passare a pyplot.grid() (default: {"which" : "both", "ls" : "dashed", "color" : "powderblue"})
 
     style_fit, style_residui : dict, optional
         dizionario contenente i parametri opzionali da passare a pyplot.errorbar() (default: {"fmt" : "."})
@@ -213,7 +230,35 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
 
     #fit
     popt, pcov = curve_fit(f_fit,t,y,sigma=dy)
-    
+    chi=sum(np.power((y-f_fit(np.array(t),*popt)), 2)/np.power(dy,2))
+    gradi_di_liberta=len(t)-len(popt)
+    chi_rid=chi/gradi_di_liberta
+
+    #valutazione grafico
+    sopra_fit=0
+    sotto_fit=0
+    lontano_fit=0
+    for element_y, element_t, element_dy in zip(y, t, dy):
+        if element_y-f_fit(element_t,*popt)>0:
+            sopra_fit+=1
+        if element_y-f_fit(element_t,*popt)<0:
+            sotto_fit+=1
+        if np.absolute(element_y-f_fit(element_t,*popt))>element_dy:
+            lontano_fit+=1
+
+    frase_log=[]
+    frase_log.append("Numero di punti sopra alla funzione di best fit: "+str(sopra_fit)+"\n"+"Numero di punti sotto alla funzione di best fit: "+str(sotto_fit))
+    frase_log.append("(expected: " + str(len(t)/2)+")")
+    frase_log.append("Numero di punti che distano dal grafico n>1 barre di errore: "+str(lontano_fit)+" (expected: "+str(0.32 *len(t))+")")
+    if log_to_file:
+        with open(path+"\\"+file_result+"_fit_log.txt", 'w') as fit_log:
+            fit_log.write("\n".join(frase_log))
+            fit_log.close()
+    else:
+        print(frase_log[0])
+        print(frase_log[1])
+        print(frase_log[2])
+
     #scrittura file_tabella.tex
     empty_space="&"*(len(popt)-1)+r"\\" #linee vuote per aggiungere spazio prima e dopo in ogni riga
     lines_prima=[r"\renewcommand{\arraystretch}{0.5}",r"\begin{table}[h!]",r"\centering"]
@@ -252,7 +297,7 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         r"\usepackage[utf8]{inputenc}",
         r"\usepackage[T1]{fontenc}",
         r"\usepackage{siunitx}",
-        r"\usepackage{amsmath}"
+        r"\usepackage{amsmath}",
         ])})
     
     if not xlabel:
@@ -261,6 +306,11 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     if not ylabel:
         ylabel=_converti_latex(variabile_y)
 
+    if x_chi==0:
+        x_chi=t[0]*1.1
+    if y_chi==0:
+        y_chi=y[0]*1.1
+
     if split_graphs:
         fig1, ax1 = plt.subplots(1, 1)
         fig2, ax2 = plt.subplots(1, 1)
@@ -300,6 +350,9 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     ax2.set_ylim(-bound, bound) # set new limits
     ax2.set_xlim(min(a1),max(a3))  
 
+    #chi^2
+    if chi:
+        ax1.text(x_chi,y_chi,r"$$\frac{\chi^2}{\nu}="+str(round(chi_rid,post_decimal_chi))+r"$$")
 
     fig1.supxlabel(xlabel, y=0.05)
     fig1.supylabel(ylabel, x=0.03)
